@@ -54,6 +54,15 @@ def show_bokeh_plot_defaults():
     pprint(plot_params.get_dict())
 
 
+def get_bokeh_help(key):
+    """
+    Print the decription of the given key in the bokeh backend
+
+    Available defaults can be seen in :py:class:`~masci_tools.vis.bokeh_plotter.BokehPlotter`
+    """
+    plot_params.get_description(key)
+
+
 ##################################### general plots ##########################
 
 
@@ -88,7 +97,10 @@ def bokeh_scatter(source,
 
     p = plot_params.prepare_figure(title, xlabel, ylabel, figure=figure)
 
-    p.scatter(x=xdata, y=ydata, source=source, **kwargs)
+    res = p.scatter(x=xdata, y=ydata, source=source, **kwargs)
+
+    if plot_params['level'] is not None:
+        res.level = plot_params['level']
 
     plot_params.draw_straight_lines(p)
     plot_params.set_limits(p)
@@ -156,9 +168,9 @@ def bokeh_multi_scatter(source,
             ydatad = []
             xdatad = []
             for i, ydat in enumerate(ydata):
-                label = 'y{}'.format(i)
+                label = f'y{i}'
                 ydatad.append(label)
-                xdatad.append('x{}'.format(i))
+                xdatad.append(f'x{i}')
                 if isinstance(xdata[0], list):
                     xdat = xdata[i]
                 else:
@@ -199,7 +211,10 @@ def bokeh_multi_scatter(source,
         if 'legend_label' not in plot_kw:
             plot_kw['legend_label'] = leg_label
 
-        p.scatter(x=xdat, y=yname, source=sourcet, **plot_kw)
+        res = p.scatter(x=xdat, y=yname, source=sourcet, **plot_kw)
+
+        if plot_params[('level', indx)] is not None:
+            res.level = plot_params[('level', indx)]
 
     plot_params.draw_straight_lines(p)
     plot_params.set_limits(p)
@@ -270,9 +285,9 @@ def bokeh_line(source,
             ydatad = []
             xdatad = []
             for i, ydat in enumerate(ydata):
-                label = 'y{}'.format(i)
+                label = f'y{i}'
                 ydatad.append(label)
-                xdatad.append('x{}'.format(i))
+                xdatad.append(f'x{i}')
                 if isinstance(xdata[0], list):
                     xdat = xdata[i]
                 else:
@@ -336,9 +351,15 @@ def bokeh_line(source,
             else:
                 p.varea(x=xdat, y1=yname, y2=shift, **kw_area, source=sourcet)
 
-        p.line(x=xdat, y=yname, source=sourcet, **kw_line, **kwargs)
+        res = p.line(x=xdat, y=yname, source=sourcet, **kw_line, **kwargs)
+        res2 = None
         if plot_points:
-            p.scatter(x=xdat, y=yname, source=sourcet, **kw_scatter)
+            res2 = p.scatter(x=xdat, y=yname, source=sourcet, **kw_scatter)
+
+        if plot_params[('level', indx)] is not None:
+            res.level = plot_params[('level', indx)]
+            if res2 is not None:
+                res2.level = plot_params[('level', indx)]
 
     plot_params.draw_straight_lines(p)
     plot_params.set_limits(p)
@@ -575,6 +596,7 @@ def bokeh_bands(bandsdata,
                 size_min=3.0,
                 size_scaling=10.0,
                 outfilename='bands_plot.html',
+                scale_color=True,
                 **kwargs):
     """
     Create an interactive bandstructure plot (non-spinpolarized) with bokeh
@@ -594,16 +616,26 @@ def bokeh_bands(bandsdata,
     :param size_min: minimum value used in scaling points for weight
     :param size_scaling: factor used in scaling points for weight
     :param outfilename: filename of the output file
+    :param scale_color: bool, if True (default) the weight will be additionally shown via a colormapping
 
     Kwargs will be passed on to :py:func:`bokeh_multi_scatter()`
     """
     from bokeh.transform import linear_cmap
 
     if weight is not None:
-        bandsdata['weight_size'] = size_min + size_scaling * bandsdata[weight] / max(bandsdata[weight])
-        plot_params.set_defaults(default_type='function',
-                                 color=linear_cmap(weight, 'Blues256', max(bandsdata[weight]), min(bandsdata[weight])),
-                                 marker_size='weight_size')
+
+        ylimits = (-15, 15)
+        if 'limits' in kwargs:
+            if 'y' in kwargs['limits']:
+                ylimits = kwargs['limits']['y']
+
+        weight_max = bandsdata[weight].loc[(bandsdata[eigenvalues] > ylimits[0]) &
+                                           (bandsdata[eigenvalues] < ylimits[1])].max()
+
+        bandsdata['weight_size'] = size_min + size_scaling * bandsdata[weight] / weight_max
+        plot_params.set_defaults(default_type='function', marker_size='weight_size')
+        if scale_color:
+            plot_params.set_defaults(default_type='function', color=linear_cmap(weight, 'Blues256', weight_max, -0.05))
     else:
         plot_params.set_defaults(default_type='function', color='black')
 
@@ -623,6 +655,7 @@ def bokeh_bands(bandsdata,
     lines = {'horizontal': 0}
     lines['vertical'] = xticks
 
+    limits = {'y': (-15, 15)}
     plot_params.set_defaults(default_type='function',
                              straight_lines=lines,
                              x_ticks=xticks,
@@ -632,7 +665,8 @@ def bokeh_bands(bandsdata,
                                  'height': 720
                              },
                              x_range_padding=0.0,
-                             y_range_padding=0.0)
+                             y_range_padding=0.0,
+                             limits=limits)
 
     return bokeh_multi_scatter(bandsdata,
                                xdata=k_label,
@@ -656,6 +690,7 @@ def bokeh_spinpol_bands(bandsdata,
                         size_min=3.0,
                         size_scaling=10.0,
                         outfilename='bands_plot.html',
+                        scale_color=True,
                         **kwargs):
     """
     Create an interactive bandstructure plot (spinpolarized) with bokeh
@@ -675,18 +710,39 @@ def bokeh_spinpol_bands(bandsdata,
     :param size_min: minimum value used in scaling points for weight
     :param size_scaling: factor used in scaling points for weight
     :param outfilename: filename of the output file
+    :param scale_color: bool, if True (default) the weight will be additionally shown via a colormapping
 
     Kwargs will be passed on to :py:func:`bokeh_multi_scatter()`
     """
     from bokeh.transform import linear_cmap
 
+    if eigenvalues is None:
+        eigenvalues = ['eigenvalues_up', 'eigenvalues_down']
+
+    plot_params.single_plot = False
+    plot_params.num_plots = 2
+
     if weight is not None:
         cmaps = ['Blues256', 'Reds256']
         color = []
+
+        ylimits = (-15, 15)
+        if 'limits' in kwargs:
+            if 'y' in kwargs['limits']:
+                ylimits = kwargs['limits']['y']
+
+        weight_max = bandsdata[weight[0]].loc[(bandsdata[eigenvalues[0]] > ylimits[0]) &
+                                              (bandsdata[eigenvalues[0]] < ylimits[1])].max()
+        weight_max = max(
+            weight_max, bandsdata[weight[1]].loc[(bandsdata[eigenvalues[1]] > ylimits[0]) &
+                                                 (bandsdata[eigenvalues[1]] < ylimits[1])].max())
+
         for indx, (w, cmap) in enumerate(zip(weight, cmaps)):
-            color.append(linear_cmap(w, cmap, max(bandsdata[w]), min(bandsdata[w])))
-            bandsdata[f'weight_size_{indx}'] = size_min + size_scaling * bandsdata[w] / max(bandsdata[w])
-        plot_params.set_defaults(default_type='function', color=color, marker_size=['weight_size_0', 'weight_size_1'])
+            color.append(linear_cmap(w, cmap, weight_max, -0.05))
+            bandsdata[f'weight_size_{indx}'] = size_min + size_scaling * bandsdata[w] / weight_max
+        plot_params.set_defaults(default_type='function', marker_size=['weight_size_0', 'weight_size_1'])
+        if scale_color:
+            plot_params.set_defaults(default_type='function', color=color)
     else:
         color = ['blue', 'red']
         plot_params.set_defaults(default_type='function', color=color)
@@ -707,21 +763,19 @@ def bokeh_spinpol_bands(bandsdata,
     lines = {'horizontal': 0}
     lines['vertical'] = xticks
 
-    plot_params.set_defaults(
-        default_type='function',
-        straight_lines=lines,
-        x_ticks=xticks,
-        x_ticklabels_overwrite=xticklabels,
-        figure_kwargs={
-            'width': 1280,
-            'height': 720
-        },
-        x_range_padding=0.0,
-        y_range_padding=0.0,
-    )
-
-    if eigenvalues is None:
-        eigenvalues = ['eigenvalues_up', 'eigenvalues_down']
+    limits = {'y': (-15, 15)}
+    plot_params.set_defaults(default_type='function',
+                             straight_lines=lines,
+                             x_ticks=xticks,
+                             x_ticklabels_overwrite=xticklabels,
+                             figure_kwargs={
+                                 'width': 1280,
+                                 'height': 720
+                             },
+                             x_range_padding=0.0,
+                             y_range_padding=0.0,
+                             limits=limits,
+                             level=[None, 'underlay'])
 
     return bokeh_multi_scatter(bandsdata,
                                xdata=k_label,
